// compute_extensions_volumeattach_v2
package testing

/*
Package testing holds fixtures (which imports testing),
so that importing volumeattach package does not inadvertently import testing into production code
More information here:
https://github.com/gophercloud/gophercloud/issues/473
*/
