// Package tokens provides information and interaction with the zone API
// resource for the OpenStack DNS service.
//
// For more information, see:
// http://developer.openstack.org/api-ref/dns/#zone
package zones
